@@ -245,6 +245,13 @@ GtkWidget *authors_page_new(GtkWidget *parent) {
   author_add(ivbox, "Marcus Bauer <marcus.bauer@gmail.com>"),
   gtk_box_pack_start(GTK_BOX(vbox), ivbox, TRUE, FALSE, 0);
 
+  /* -------------------------------------------- */
+  ivbox = gtk_vbox_new(FALSE, 0);
+  author_add(ivbox, _("Patches/bug fixes by:"));
+  author_add(ivbox, "Roman Kapusta");
+  author_add(ivbox, "Charles Clément");
+  gtk_box_pack_start(GTK_BOX(vbox), ivbox, TRUE, FALSE, 0);
+
   /* -------------------------------------------- */
   ivbox = gtk_vbox_new(FALSE, 0);
   author_add(ivbox, _("Search and wikipedia feature provided by:"));
