@@ -559,14 +559,16 @@ void geonames_enable_search(GtkWidget *map) {
   g_object_set_data(G_OBJECT(toplevel), GEONAMES_SEARCH, hbox);
 }
 
-static void wiki_context_free(GtkWidget *widget) {
+static void wiki_context_free(GtkWidget *widget, gboolean destroy) {
   wiki_context_t *context = 
     (wiki_context_t *)g_object_get_data(G_OBJECT(widget), "wikipedia");
 
   g_assert(context);
 
-  osm_gps_map_clear_images(OSM_GPS_MAP(widget));
-  osm_gps_map_osd_clear_balloon (OSM_GPS_MAP(widget));
+  if(!destroy) {
+    osm_gps_map_clear_images(OSM_GPS_MAP(widget));
+    osm_gps_map_osd_clear_balloon (OSM_GPS_MAP(widget));
+  }
 
   /* if a list of entries is present, then remove it */
   if(context->list) {
@@ -615,7 +617,7 @@ static void wiki_context_free(GtkWidget *widget) {
 }
 
 static void on_map_destroy (GtkWidget *widget, gpointer data) {
-  wiki_context_free(widget);
+  wiki_context_free(widget, TRUE);
 }
 
 static void geonames_entry_render(gpointer data, gpointer user_data) {
@@ -739,7 +741,8 @@ static void wikipedia_details(GtkWidget *map, geonames_entry_t *entry) {
 
     /* scrolling text window */
     /* put this inside a scrolled view */
-    GtkWidget *scrolled_window = scrolled_window_new();
+    GtkWidget *scrolled_window = 
+      scrolled_window_new(GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
     scrolled_window_add_with_viewport(scrolled_window, label);
     gtk_box_pack_start_defaults(GTK_BOX(vbox), scrolled_window);
   }
@@ -1058,7 +1061,7 @@ void geonames_enable_wikipedia(GtkWidget *map, gboolean enable) {
       wiki_update(map);
 
   } else
-    wiki_context_free(map);
+    wiki_context_free(map, FALSE);
 
   gconf_set_bool("wikipedia", enable);
 }
