@@ -19,12 +19,14 @@
 
 #ifdef USE_MAEMO
 #include <libosso.h>
+#include <hildon/hildon-banner.h>
 #endif
 
 #include "config.h"
 #include "menu.h"
 #include "track.h"
 #include "geonames.h"
+#include "hxm.h"
 
 #ifdef MAEMO5
 #include <gdk/gdkx.h>
@@ -53,6 +55,7 @@
 #include "osm-gps-map.h"
 #include "osm-gps-map-osd-classic.h"
 #include "converter.h"
+#include "misc.h"
 
 #include "gps.h"
 
@@ -103,49 +106,33 @@ static const char *get_proxy_uri(void) {
 }
 
 /* the gconf keys the map state will be stored under */
-#define GCONF_PATH           "/apps/maep/"
-#define GCONF_KEY_ZOOM       GCONF_PATH "zoom"
-#define GCONF_KEY_SOURCE     GCONF_PATH "source"
-#define GCONF_KEY_LATITUDE   GCONF_PATH "latitude"
-#define GCONF_KEY_LONGITUDE  GCONF_PATH "longitude"
-
-static gint 
-gconf_get_int(GConfClient *client, char *key, gint def_value) {
-  GConfValue *value = gconf_client_get(client, key, NULL);
-
-  if(!value) return def_value;
-
-  return gconf_client_get_int(client, key, NULL);
-}
-
-static gfloat 
-gconf_get_float(GConfClient *client, char *key, gfloat def_value) {
-  GConfValue *value = gconf_client_get(client, key, NULL);
-
-  if(!value) return def_value;
-
-  return gconf_client_get_float(client, key, NULL);
-}
+#define GCONF_KEY_ZOOM       "zoom"
+#define GCONF_KEY_SOURCE     "source"
+#define GCONF_KEY_LATITUDE   "latitude"
+#define GCONF_KEY_LONGITUDE  "longitude"
+#define GCONF_KEY_DOUBLEPIX  "double-pixel"
 
 /* save the entire map state into gconf to be able */
 /* to restore the state at the next startup */
 void map_save_state(GtkWidget *widget) {
   gint zoom, source;
   gfloat lat, lon;
+  gboolean dpix;
 
   /* get state information from map ... */
   g_object_get(OSM_GPS_MAP(widget), 
 	       "zoom", &zoom, 
 	       "map-source", &source, 
 	       "latitude", &lat, "longitude", &lon,
+	       "double-pixel", &dpix,
 	       NULL);
 
   /* ... and store it in gconf */
-  GConfClient *gconf_client = gconf_client_get_default();
-  gconf_client_set_int(gconf_client, GCONF_KEY_ZOOM, zoom, NULL);
-  gconf_client_set_int(gconf_client, GCONF_KEY_SOURCE, source, NULL);
-  gconf_client_set_float(gconf_client, GCONF_KEY_LATITUDE, lat, NULL);
-  gconf_client_set_float(gconf_client, GCONF_KEY_LONGITUDE, lon, NULL);
+  gconf_set_int(GCONF_KEY_ZOOM, zoom);
+  gconf_set_int(GCONF_KEY_SOURCE, source);
+  gconf_set_float(GCONF_KEY_LATITUDE, lat);
+  gconf_set_float(GCONF_KEY_LONGITUDE, lon);
+  gconf_set_bool(GCONF_KEY_DOUBLEPIX, dpix);
 }
 
 static int dist2pixel(OsmGpsMap *map, float km) {
@@ -155,7 +142,7 @@ static int dist2pixel(OsmGpsMap *map, float km) {
 static void
 cb_map_gps(osd_button_t but, void *data) {
   OsmGpsMap *map = OSM_GPS_MAP(data);
-  struct gps_fix_t *fix = g_object_get_data(G_OBJECT(map), "gps_fix");
+  struct gps_t *fix = g_object_get_data(G_OBJECT(map), "gps_fix");
 
   if(but == OSD_GPS && fix) {
 
@@ -174,7 +161,7 @@ static float get_distance(coord_t *p1, coord_t *p2) {
 }
 #endif
 
-static void gps_callback(gps_mask_t set, struct gps_fix_t *fix, void *data) {
+static void gps_callback(gps_mask_t set, struct gps_t *fix, void *data) {
   OsmGpsMap *map = OSM_GPS_MAP(data);
 #ifdef USE_MAEMO
   GtkWidget *toplevel = gtk_widget_get_toplevel(GTK_WIDGET(map));
@@ -184,19 +171,19 @@ static void gps_callback(gps_mask_t set, struct gps_fix_t *fix, void *data) {
     (gps_mask_t)g_object_get_data(G_OBJECT(map), "gps_set"); 
 
   /* ... and enable "goto" button if it's valid */
-  if((set & LATLON_SET) != (gps_set & LATLON_SET)) {
+  if((set & FIX_LATLON_SET) != (gps_set & FIX_LATLON_SET)) {
     osm_gps_map_osd_enable_gps(map,  
-       OSM_GPS_MAP_OSD_CALLBACK((set&LATLON_SET)?cb_map_gps:NULL), map);
+       OSM_GPS_MAP_OSD_CALLBACK((set&FIX_LATLON_SET)?cb_map_gps:NULL), map);
 
     g_object_set_data(G_OBJECT(map), "gps_set", (gpointer)set); 
   }
 
-  if(set & LATLON_SET) {
+  if(set & FIX_LATLON_SET) {
     /* save fix in case the user later wants to enable gps */
     g_object_set_data(G_OBJECT(map), "gps_fix", fix); 
 
     /* get error */
-    int radius = (set & HERR_SET)?dist2pixel(map, fix->eph/1000):0;
+    int radius = (set & FIX_HERR_SET)?dist2pixel(map, fix->eph/1000):0;
 
     g_object_set(map, "gps-track-highlight-radius", radius, NULL);
     osm_gps_map_draw_gps(map, fix->latitude, fix->longitude, fix->track);
@@ -207,54 +194,73 @@ static void gps_callback(gps_mask_t set, struct gps_fix_t *fix, void *data) {
   }
 
 #ifdef USE_MAEMO
-  if((set & LATLON_SET) && toplevel) {
-
-    /* check if toplevel has focus */
-    if(GTK_WIDGET_HAS_FOCUS(toplevel)) {
-
-      /* check gps position every 10 seconds and trigger */
-      /* screen saver if useful */
-      time_t last = (time_t)g_object_get_data(G_OBJECT(map), "gps_timer"); 
-      time_t now = time(NULL);
-      if(now-last > 10) {
-	/* get last saved position */
-	coord_t *last_pos = g_object_get_data(G_OBJECT(map), "gps_pos"); 
-	coord_t cur = { .rlat = deg2rad(fix->latitude),
-			.rlon = deg2rad(fix->longitude) };
+  if((set & FIX_LATLON_SET) && toplevel) {
+    /* check gps position every 10 seconds and trigger */
+    /* screen saver if useful */
+    time_t last = (time_t)g_object_get_data(G_OBJECT(map), "gps_timer"); 
+    time_t now = time(NULL);
+    if(now-last > 10) {
+      /* get last saved position */
+      coord_t *last_pos = g_object_get_data(G_OBJECT(map), "gps_pos"); 
+      coord_t cur = { .rlat = deg2rad(fix->latitude),
+		      .rlon = deg2rad(fix->longitude) };
+      
+      if(last_pos) {
+	/* consider everything under 3 kph (~2 mph) to be static and */
+	/* disable (trigger) screensaver only above this. 3.6kph = 10m/10sec */
 	
-	if(last_pos) {
-	  /* consider everything under 3 kph (~2 mph) to be static and */
-	  /* disable (trigger) screensaver only above this. 3kph = ~8m/10sec */
+	/* compare with reported position */
+	if(get_distance(last_pos, &cur) > (now-last)) {
+	  printf("SCREENSAVER: trigger after %d ...\n", (int)(now-last));
+	  
+	  osso_context_t *osso_context =  
+	    g_object_get_data(G_OBJECT(toplevel), "osso-context");
+	  g_assert(osso_context);
+	  
+	  osso_display_blanking_pause(osso_context);
 	  
-	  /* compare with reported position */
-	  if(get_distance(last_pos, &cur) > 8) {
-	    printf("trigger screen saver after %d ...\n", (int)(now-last));
-	    
-	    osso_context_t *osso_context =  
-	      g_object_get_data(G_OBJECT(toplevel), "osso-context");
-	    g_assert(osso_context);
-	    
-	    osso_display_blanking_pause(osso_context);
-	    
-	  } else
-	    printf("too slow, no trigger\n");
 	} else
-	  last_pos = g_new0(coord_t, 1);
-	
-	/* make current position the old one */
-	last_pos->rlat = cur.rlat;
-	last_pos->rlon = cur.rlon;
-	
-	g_object_set_data(G_OBJECT(map), "gps_timer", (gpointer)now); 
-	g_object_set_data(G_OBJECT(map), "gps_pos", last_pos); 
+	  printf("SCREENSAVER: too slow, no trigger\n");
+      } else {
+	printf("SCREENSAVER: no last pos\n");
+	last_pos = g_new0(coord_t, 1);
       }
+      
+      /* make current position the old one */
+      last_pos->rlat = cur.rlat;
+      last_pos->rlon = cur.rlon;
+      
+      g_object_set_data(G_OBJECT(map), "gps_timer", (gpointer)now); 
+      g_object_set_data(G_OBJECT(map), "gps_pos", last_pos); 
     }
   }
 #endif
 
 }
 
-static GtkWidget *map_new(void) {
+static gboolean on_focus_change(GtkWidget *widget, GdkEventFocus *event,
+				gpointer user_data) {
+  GtkWidget *map = GTK_WIDGET(user_data);
+
+  gps_state_t *gps = g_object_get_data(G_OBJECT(map), "gps_state");
+  g_assert(gps);
+
+  printf("map focus-%s event\n", event->in?"in":"out");
+
+  /* disconnect from gps if map looses focus */
+  /* this is to save energy if maep runs in background */
+  
+  if(event->in) {
+    /* request all GPS information required for map display */
+    gps_register_callback(gps, LATLON_CHANGED | TRACK_CHANGED | HERR_CHANGED, 
+			  gps_callback, map);
+  } else
+    gps_unregister_callback(gps, gps_callback);
+
+  return TRUE;
+}
+
+static GtkWidget *map_new(GtkWidget *window) {
   /* It is recommanded that all applications share these same */
   /* map path, so data is only cached once. The path should be: */
   /* ~/.osm-gps-map on standard PC     (users home) */
@@ -281,26 +287,28 @@ static GtkWidget *map_new(void) {
   path = g_strdup("/media/mmc2/osm-gps-map");
 #endif
 #endif
-  GConfClient *gconf_client = gconf_client_get_default();
-  gint source = gconf_get_int(gconf_client, GCONF_KEY_SOURCE, MAP_SOURCE);
+  gint source = gconf_get_int(GCONF_KEY_SOURCE, MAP_SOURCE);
 
   const char *proxy = get_proxy_uri();
 
   /* get zoom, latitude and longitude from gconf if possible */
-  gint zoom = gconf_get_int(gconf_client, GCONF_KEY_ZOOM, 3);
-  gfloat lat = gconf_get_float(gconf_client, GCONF_KEY_LATITUDE, 50.0);
-  gfloat lon = gconf_get_float(gconf_client, GCONF_KEY_LONGITUDE, 21.0);
+  gint zoom = gconf_get_int(GCONF_KEY_ZOOM, 3);
+  gfloat lat = gconf_get_float(GCONF_KEY_LATITUDE, 50.0);
+  gfloat lon = gconf_get_float(GCONF_KEY_LONGITUDE, 21.0);
+  gboolean dpix = gconf_get_bool(GCONF_KEY_DOUBLEPIX, FALSE);
     
   GtkWidget *widget = g_object_new(OSM_TYPE_GPS_MAP,
 		 "map-source",               source,
 		 "tile-cache",               OSM_GPS_MAP_CACHE_FRIENDLY,
-                 "tile-cache-base",          path,
+		 "tile-cache-base",          path,
 		 "auto-center",              FALSE,
 		 "record-trip-history",      FALSE, 
 		 "show-trip-history",        FALSE, 
 		 "gps-track-point-radius",   10,
 		 proxy?"proxy-uri":NULL,     proxy,
-                 NULL);
+		 "drag-limit",               MAP_DRAG_LIMIT,
+		 "double-pixel",             dpix,
+		 NULL);
 
   g_free(path);
 
@@ -338,9 +346,14 @@ static GtkWidget *map_new(void) {
   /* request all GPS information required for map display */
   gps_register_callback(gps, LATLON_CHANGED | TRACK_CHANGED | HERR_CHANGED, 
 			gps_callback, widget);
-
   g_object_set_data(G_OBJECT(widget), "gps_state", gps);
 
+  g_signal_connect(G_OBJECT(window), "focus-in-event", 
+		   G_CALLBACK(on_focus_change), widget);
+  
+  g_signal_connect(G_OBJECT(window), "focus-out-event", 
+		   G_CALLBACK(on_focus_change), widget);
+  
   return widget;
 }
 
@@ -375,6 +388,60 @@ static void on_window_destroy (GtkWidget *widget, gpointer data) {
   gtk_main_quit();
 }
 
+static void hxm_callback(hxm_t *hxm, void *data) {
+  GtkWidget *map = GTK_WIDGET(data);
+
+  switch(hxm->state) {
+  case HXM_STATE_CONNECTED:
+    osm_gps_map_osd_draw_hr(OSM_GPS_MAP(map), TRUE, hxm->hr);
+    break;
+  case HXM_STATE_FAILED:
+#ifdef USE_MAEMO
+    {
+      GtkWidget *toplevel = gtk_widget_get_toplevel(GTK_WIDGET(map));
+      hildon_banner_show_information(toplevel, NULL, 
+		   _("Connection to heart rate monitor failed!"));
+    }
+#endif
+    osm_gps_map_osd_draw_hr(OSM_GPS_MAP(map), FALSE, OSD_HR_ERROR);
+    break;
+  default:
+    osm_gps_map_osd_draw_hr(OSM_GPS_MAP(map), FALSE, OSD_HR_INVALID);
+    break;
+  }
+}
+
+void hxm_enable(GtkWidget *map, gboolean enable) {
+  printf("%sabling heart rate capture\n", enable?"en":"dis");
+
+  /* verify that tracking isn't already in the requested state */
+  gboolean cur_state = 
+    (gboolean)g_object_get_data(G_OBJECT(map), HXM_ENABLED);
+
+  g_assert(cur_state != enable);
+
+  /* save new tracking state */
+  g_object_set_data(G_OBJECT(map), HXM_ENABLED, (gpointer)enable);
+
+  hxm_t *hxm = g_object_get_data(G_OBJECT(map), "hxm");
+
+  if(enable) {
+    g_assert(!hxm);
+
+    hxm = hxm_init();
+    hxm_register_callback(hxm, hxm_callback, map);
+    g_object_set_data(G_OBJECT(map), "hxm", hxm);
+  } else {
+    g_assert(hxm);
+
+    g_object_set_data(G_OBJECT(map), "hxm", NULL);
+    hxm_release(hxm);
+
+    /* undraw gui */
+    osm_gps_map_osd_draw_hr(OSM_GPS_MAP(map), FALSE, OSD_HR_NONE);
+  }
+}
+
 static void on_map_destroy (GtkWidget *widget, gpointer data) {
   track_save(widget);
 
@@ -385,11 +452,40 @@ static void on_map_destroy (GtkWidget *widget, gpointer data) {
   g_assert(state);
   gps_release(state);
 
+  hxm_t *hxm = g_object_get_data(G_OBJECT(widget), "hxm");
+  if(hxm) {
+    g_object_set_data(G_OBJECT(widget), "hxm", NULL);
+    hxm_release(hxm);
+  }
+
+  gconf_set_bool(HXM_ENABLED, 
+    (gboolean)g_object_get_data(G_OBJECT(widget), HXM_ENABLED));
+
   map_save_state(widget);
 }
 
-int main(int argc, char *argv[]) {
+static gboolean 
+on_window_key_press(GtkWidget *window, GdkEventKey *event, GtkWidget *map) 
+{
+#ifdef USE_MAEMO
+  if((event->keyval == HILDON_HARDKEY_FULLSCREEN) || 
+     (event->keyval == HILDON_HARDKEY_INCREASE) || 
+     (event->keyval == HILDON_HARDKEY_DECREASE)) 
+#else
+  if(event->keyval == GDK_F11) 
+#endif
+  {
+    gboolean return_val;
+    g_signal_emit_by_name(GTK_OBJECT(map), "key_press_event", 
+			  event, &return_val);
+    return return_val;
+  }
+  
+  return FALSE;
+}
 
+int main(int argc, char *argv[]) {
+  
 #if 0
   setlocale(LC_ALL, "");
   bindtextdomain(PACKAGE, LOCALEDIR);
@@ -445,7 +541,7 @@ int main(int argc, char *argv[]) {
   gtk_container_add(GTK_CONTAINER(window), vbox);
 
   /* create map widget */
-  GtkWidget *map = map_new();
+  GtkWidget *map = map_new(window);
 
   /* attach menu to main window */
   menu_create(vbox, map);
@@ -455,10 +551,19 @@ int main(int argc, char *argv[]) {
 
   gtk_box_pack_start_defaults(GTK_BOX(vbox), map);
 
-  
   track_restore(map);
   geonames_wikipedia_restore(map);
 
+  /* heart rate data */
+  if(gconf_get_bool(HXM_ENABLED, FALSE)) 
+    menu_check_set_active(window, "Heart Rate", TRUE);
+  
+  /* connect a key handler to forward global shortcuts (function keys) */
+  /* to the ap widget */
+  g_signal_connect(G_OBJECT(window), "key_press_event",
+		   G_CALLBACK(on_window_key_press), map);
+
+
   gtk_widget_show_all(GTK_WIDGET(window));
   gtk_main();
 
