@@ -114,6 +114,50 @@ gboolean gconf_get_bool(char *m_key, gboolean default_value) {
   return ret;
 }
 
+void gconf_set_int(char *m_key, gint value) {
+  GConfClient *client = gconf_client_get_default();
+  char *key = g_strdup_printf(GCONF_PATH, m_key);
+  gconf_client_set_int(client, key, value, NULL);
+  g_free(key);
+}
+
+gint gconf_get_int(char *m_key, gint def_value) {
+  GConfClient *client = gconf_client_get_default();
+
+  char *key = g_strdup_printf(GCONF_PATH, m_key);
+  GConfValue *value = gconf_client_get(client, key, NULL);
+  if(!value) {
+    g_free(key);
+    return def_value;
+  }
+
+  gint ret = gconf_client_get_int(client, key, NULL);
+  g_free(key);
+  return ret;
+}
+
+void gconf_set_float(char *m_key, gfloat value) {
+  GConfClient *client = gconf_client_get_default();
+  char *key = g_strdup_printf(GCONF_PATH, m_key);
+  gconf_client_set_float(client, key, value, NULL);
+  g_free(key);
+}
+
+gfloat gconf_get_float(char *m_key, gfloat def_value) {
+  GConfClient *client = gconf_client_get_default();
+
+  char *key = g_strdup_printf(GCONF_PATH, m_key);
+  GConfValue *value = gconf_client_get(client, key, NULL);
+  if(!value) {
+    g_free(key);
+    return def_value;
+  }
+
+  gfloat ret = gconf_client_get_float(client, key, NULL);
+  g_free(key);
+  return ret;
+}
+
 static const char *data_paths[] = {
   "~/." APP,                 // in home directory
   DATADIR ,                  // final installation path (e.g. /usr/share/maep)
@@ -356,6 +400,18 @@ GtkWidget *label_big_new(char *str) {
   return label;
 }
 
+GtkWidget *label_left_new(char *str) {
+  GtkWidget *widget = gtk_label_new(str);
+  gtk_misc_set_alignment(GTK_MISC(widget), 0.0f, 0.5f);
+  return widget;
+}
+
+GtkWidget *label_right_new(char *str) {
+  GtkWidget *widget = gtk_label_new(str);
+  gtk_misc_set_alignment(GTK_MISC(widget), 1.0f, 0.5f);
+  return widget;
+}
+
 static void  
 on_label_realize(GtkWidget *widget, gpointer user_data)  {
   /* get parent size (which is a container) */
@@ -375,13 +431,19 @@ GtkWidget *label_wrap_new(char *str) {
 }
 
 /* create a pannable area */
-GtkWidget *scrolled_window_new(void) {
+GtkWidget *scrolled_window_new(GtkPolicyType hp,  GtkPolicyType vp) {
 #ifdef MAEMO5
-  return hildon_pannable_area_new();
+  HildonMovementMode mode = 0;
+  GtkWidget *pannable_area = hildon_pannable_area_new();
+  if(hp != GTK_POLICY_NEVER) mode |= HILDON_MOVEMENT_MODE_HORIZ;
+  if(vp != GTK_POLICY_NEVER) mode |= HILDON_MOVEMENT_MODE_VERT;
+
+  g_object_set(G_OBJECT(pannable_area), "mov-mode", mode, NULL);
+
+  return pannable_area;
 #else
   GtkWidget *scrolled_window = gtk_scrolled_window_new(NULL, NULL);
-  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
-  				 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), hp, vp);
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_window), 
 				      GTK_SHADOW_NONE);
   return scrolled_window;
@@ -398,3 +460,7 @@ void scrolled_window_add_with_viewport(GtkWidget *win, GtkWidget *child) {
 #endif
 }
 
+void scrolled_window_add(GtkWidget *win, GtkWidget *child) {
+  gtk_container_add(GTK_CONTAINER(win), child);
+}
+
